from typing import List, Dict
from datetime import date
import traceback
import requests
from dateutil.parser import parse as parse_datetime, isoparse
import re
from elasticsearch7 import Elasticsearch
from elasticsearch7.helpers import scan

es = Elasticsearch(["http://es-oscars-01.vgt.vito.be:9200", "http://es-oscars-02.vgt.vito.be:9200",
                    "http://es-oscars-03.vgt.vito.be:9200"])

layers = [
        # (PDF catalog product type, TSService layer name, OSCARS index, Latest date allowed to be missing?) pairs
        # ('RAINFALL', 'CHIRPS_RAINFALL', False), # not exposed by PDFCatalog
        ('PROBAV_L3_S10_TOC_333M', 'PROBAV_L3_S10_TOC_NDVI_333M', 'oscars_probav_s10toc333m_v1', True),
        # ('PROBAV_L3_S10_TOC_333M', 'PROBAV_L3_S10_TOC_SM_333M', False), # not in Accumulo
        # ('CGS_S2_FAPAR', 'TERRASCOPE_S2_FAPAR_V2', False),
        # ('CGS_S2_FAPAR', 'S2_SCENECLASSIFICATION_V200_FILE', False),
        # ('CGS_S2_FCOVER', 'TERRASCOPE_S2_FCOVER_V2', False),
        # ('CGS_S2_LAI', 'TERRASCOPE_S2_LAI_V2', False),
        # ('CGS_S2_NDVI', 'TERRASCOPE_S2_NDVI_V2', False),

        # "('CGS_S1_GRD_SIGMA0_L1', 'S1_GRD_SIGMA0_DESCENDING_VH', 'DESCENDING', False),

        # ('CGS_S1_GRD_SIGMA0_L1', 'S1_GRD_SIGMA0_ASCENDING_VH', 'ASCENDING', False),

        # note: currently multiband layers are not being returned in tsservice layer metadata so use 'S1_GRD_GAMMA0_VH'
        # instead of 'S1_GRD_GAMMA0'
        ('CGS_S1_GRD_SIGMA0_L1', 'S1_GRD_GAMMA0_VH', 'oscars_cgss1_grd_sigma0_l1', True)

        # ('???', 'S2A_31UGU_FAPAR', False), # not in PDF yet
        # ('???', 'S2A_Emilia-Romagna_FAPAR_NewTiles', False), # not in PDF yet
        # ('???', 'S2A_32TML_FAPAR', False), # not in PDF yet

        # ('', 'FSTEP_METEO_RAINFALL', False), # not in PDF
        # ('', 'FSTEP_METEO_WEEKLY_RAINFALL', False), # not in PDF
        # ('', 'FSTEP_METEO_MAX_TEMPERATURE', False), # not in PDF
        # ('', 'FSTEP_METEO_MIN_TEMPERATURE', False), # not in PDF
        # ('', 'FSTEP_METEO_AVG_TEMPERATURE', False), # not in PDF
        # ('', 'FSTEP_METEO_WEEKLY_AVG_TEMPERATURE', False), # not in PDF

        # ('NEXTGEOSS_SENTINEL2_FAPAR', 'NEXTGEOSS_SENTINEL2_FAPAR', False), # not in Accumulo
        # ('NEXTGEOSS_SENTINEL2_LAI', 'NEXTGEOSS_SENTINEL2_LAI', False), # not in Accumulo
        # ('NEXTGEOSS_SENTINEL2_NDVI', 'NEXTGEOSS_SENTINEL2_NDVI', False), # not in Accumulo,
        # ('NEXTGEOSS_SENTINEL2_FCOVER', 'NEXTGEOSS_SENTINEL2_FCOVER', False), # not in Accumulo
        # ('FSTEP_SENTINEL2_FAPAR', 'FSTEP_SENTINEL2_FAPAR', False), # not in Accumulo
        # ('FSTEP_SENTINEL2_LAI', 'FSTEP_SENTINEL2_LAI', False), # not in Accumulo
        # ('FSTEP_SENTINEL2_NDVI', 'FSTEP_SENTINEL2_NDVI', False), # not in Accumulo
        # ('FSTEP_SENTINEL2_FCOVER', 'FSTEP_SENTINEL2_FCOVER', False) # not in Accumulo
    ]


def parse_date(timestr: str) -> date:
    match = re.fullmatch(r"^(.+)\[.+\]$", timestr)  # workaround for tsservice returning the time zone for V200 layers
    return parse_datetime(match.group(1) if match else timestr).date()


def get_dates_from_catalogue(product_index: str, from_year: int = None) -> List[date]:
    if False and from_year is not None:
        r = scan(
            es,
            index=product_index,
            q=f"properties.acquisitionInformation.acquisitionParameters.beginningDateTime: [{str(from_year)} TO *]",
            _source="properties.acquisitionInformation.acquisitionParameters.beginningDateTime"
        )
    else:
        r = scan(
            es,
            index=product_index,
            _source="properties.acquisitionInformation.acquisitionParameters.beginningDateTime"
        )

    timestrs = [
        next(
            filter(
                lambda x: 'acquisitionParameters' in x,
                doc['_source']['properties']['acquisitionInformation']
            )
        )['acquisitionParameters']['beginningDateTime']
        for doc in r
    ]
    timestamps = [parse_date(timestr) for timestr in timestrs]
    return sorted(timestamps)


def get_dates_from_time_series_service(layer_name: str) -> List[date]:
    r = requests.get("https://proba-v-mep.esa.int/api/timeseries/v1.0/ts")
    r.raise_for_status()

    layer = [l for l in r.json()["layers"] if l["name"] == layer_name][0]
    timestamps = [parse_date(d) for d in layer["dates"]]
    return timestamps


def main():
    missing_dates_found = False

    try:
        for product_type, layer_name, oscars_index, latest_missing_ok, *args in layers:
            expected_dates = get_dates_from_catalogue(oscars_index, 2017)

            if latest_missing_ok:
                expected_dates = expected_dates[:-1]

            actual_dates = get_dates_from_time_series_service(layer_name)

            # print(str(product_type) + " expected_dates: " + str(expected_dates))
            # print(str(product_type) + " actual_dates: " + str(actual_dates))

            missing_dates = sorted(list(set(expected_dates) - set(actual_dates)))

            for missing_date in missing_dates:
                print("WARNING: Catalogue's", product_type, "/", missing_date, "is not in Geotrellis'", layer_name)

            if missing_dates:
                missing_dates_found = True
    except:
        traceback.print_exc()
        exit(2)

    if missing_dates_found:
        exit(1)
    else:
        print('OK: No missing dates found')


if __name__ == '__main__':
    main()
